
/**********************************************************************************************/
/*Library Management Tables Script.sql*/
/**********************************************************************************************/
/*--------------------------------------------------------------------------------------------
Last Updated Date:	2012.03.23 
Last Updated By:	hans.p.halvorsen@hit.no
Description:		Library Management System

(C) Hans-Petter Halvorsen
-------------------------------------------------------------------------------------------*/

/*
LIBRARY
AUTHOR
PUBLISHER
CATEGORY
BOOK
BOOK_LIBRARY
CHAPTER
CUSTOMER
LOAN
RATING
*/

if not exists (select * from dbo.sysobjects where id = object_id(N'[LIBRARY]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [LIBRARY]
(
	[LibraryId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[LibraryName] [varchar](50) NOT NULL UNIQUE,
	[Description] [varchar](1000) NULL,
	[Address] [varchar](50) NULL,
	[Phone] [varchar](50) NULL,
	[PostCode] [varchar](50) NULL,
	[PostAddress] [varchar](50) NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[AUTHOR]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [AUTHOR]
(
	[AuthorId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[AuthorName] [varchar](50) NOT NULL UNIQUE,
	[Address] [varchar](50) NULL,
	[Phone] [varchar](50) NULL,
	[PostCode] [varchar](50) NULL,
	[PostAddress] [varchar](50) NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[PUBLISHER]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [PUBLISHER]
(
	[PublisherId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[PublisherName] [varchar](50) NOT NULL UNIQUE,
	[Description] [varchar](1000) NULL,
	[Address] [varchar](50) NULL,
	[Phone] [varchar](50) NULL,
	[PostCode] [varchar](50) NULL,
	[PostAddress] [varchar](50) NULL,
	[EMail] [varchar](50) NULL,	
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[CATEGORY]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [CATEGORY]
(
	[CategoryId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[CategoryName] [varchar](50) NOT NULL UNIQUE,
	[Description] [varchar](1000) NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[BOOK]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [BOOK]
(
	[BookId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[Title] [varchar](50) NOT NULL UNIQUE,
	[ISBN] [varchar](20) NOT NULL,
	[PublisherId] [int] NOT NULL FOREIGN KEY REFERENCES [PUBLISHER] ([PublisherId]),
	[AuthorId] [int] NOT NULL FOREIGN KEY REFERENCES [AUTHOR] ([AuthorId]),
	[CategoryId] [int] NOT NULL FOREIGN KEY REFERENCES [CATEGORY] ([CategoryId]),
	[Description] [varchar](1000) NULL,
	[Year] [date] NULL,
	[Edition] [int] NULL,
	[AverageRating] [float] NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[BOOK_LIBRARY]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [BOOK_LIBRARY]
(
	[BookLibraryId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[BookId] [int] NOT NULL FOREIGN KEY REFERENCES [BOOK] ([BookId]),
	[LibraryId] [int] NOT NULL FOREIGN KEY REFERENCES [LIBRARY] ([LibraryId]),
	[OnLoan] [bit] NOT NULL DEFAULT(0),
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[CHAPTER]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [CHAPTER]
(
	[ChapterId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[ChapterNumber] [int] NOT NULL,
	[ChapterName] [varchar](50) NOT NULL,
	[BookId] [int] NOT NULL FOREIGN KEY REFERENCES [BOOK] ([BookId]),
	[Description] [varchar](1000) NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[CUSTOMER]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [CUSTOMER]
(
	[CustomerId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[CustomerName] [varchar](50) NOT NULL,
	[Address] [varchar](50) NULL,
	[Phone] [varchar](50) NULL,
	[PostCode] [varchar](50) NULL,
	[PostAddress] [varchar](50) NULL,
	[EMail] [varchar](50) NULL,	
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[LOAN]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [LOAN]
(
	[LoanId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[BookId] [int] NOT NULL FOREIGN KEY REFERENCES [BOOK] ([BookId]),
	[CustomerId] [int] NOT NULL FOREIGN KEY REFERENCES [CUSTOMER] ([CustomerId]),
	[LibraryId] [int] NOT NULL FOREIGN KEY REFERENCES [LIBRARY] ([LibraryId]),
	[LoanDate] [datetime] NOT NULL,
	[Deadline] [datetime] NULL,
	[Submitted] [bit] NULL,
	[SubmittedDate] [datetime] NULL,
) 
GO



if not exists (select * from dbo.sysobjects where id = object_id(N'[RATING]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [RATING]
(
	[RatingId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[BookId] [int] NOT NULL FOREIGN KEY REFERENCES [BOOK] ([BookId]),
	[CustomerId] [int] NOT NULL FOREIGN KEY REFERENCES [CUSTOMER] ([CustomerId]),
	[RatingDate] [datetime] NOT NULL,
	[Score] [float] NOT NULL,
	[Comment] [varchar](1000) NULL,
) 
GO




	
/**********************************************************************************************/
/*GetBookData.view.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'GetBookData' 
	   AND 	  type = 'V')
	DROP VIEW GetBookData
GO

CREATE VIEW GetBookData
AS

SELECT
BOOK.BookId, 
BOOK.Title, 
BOOK.ISBN, 
PUBLISHER.PublisherName, 
AUTHOR.AuthorName, 
CATEGORY.CategoryName,
BOOK.AverageRating,
BOOK_LIBRARY.OnLoan,
LIBRARY.LibraryId

FROM         
BOOK 
INNER JOIN BOOK_LIBRARY ON BOOK.BookId = BOOK_LIBRARY.BookId 
INNER JOIN LIBRARY ON BOOK_LIBRARY.LibraryId = LIBRARY.LibraryId 
INNER JOIN AUTHOR ON BOOK.AuthorId = AUTHOR.AuthorId 
INNER JOIN PUBLISHER ON BOOK.PublisherId = PUBLISHER.PublisherId 
INNER JOIN CATEGORY ON BOOK.CategoryId = CATEGORY.CategoryId


GO
	
/**********************************************************************************************/
/*GetBookLoan.view.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'GetBookLoan' 
	   AND 	  type = 'V')
	DROP VIEW GetBookLoan
GO

CREATE VIEW GetBookLoan
AS


SELECT
BOOK.Title, 
BOOK.BookId, 
CUSTOMER.CustomerId, 
CUSTOMER.CustomerName, 
LOAN.LoanDate, 
LOAN.Deadline, 
LOAN.Submitted,
LOAN.SubmittedDate

FROM
BOOK 
INNER JOIN LOAN ON BOOK.BookId = LOAN.BookId 
INNER JOIN CUSTOMER ON LOAN.CustomerId = CUSTOMER.CustomerId


GO
	
/**********************************************************************************************/
/*CreateBook.sp.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'CreateBook' 
	   AND 	  type = 'P')
	DROP PROCEDURE CreateBook
GO

CREATE PROCEDURE CreateBook
@Title varchar(50),
@Isbn varchar(20),
@PublisherName varchar(50),
@AuthorName varchar(50),
@CategoryName varchar(50)
AS



if not exists (select * from CATEGORY where CategoryName = @CategoryName)
	INSERT INTO CATEGORY (CategoryName) VALUES (@CategoryName)

if not exists (select * from AUTHOR where AuthorName = @AuthorName)
	INSERT INTO AUTHOR (AuthorName) VALUES (@AuthorName)

if not exists (select * from PUBLISHER where PublisherName = @PublisherName)
	INSERT INTO PUBLISHER (PublisherName) VALUES (@PublisherName)


if not exists (select * from BOOK where Title = @Title)
	INSERT INTO BOOK (Title, ISBN, PublisherId, AuthorId, CategoryId) 
	VALUES 
	(
	@Title,
	@ISBN, 
	(select PublisherId from PUBLISHER where PublisherName=@PublisherName),
	(select AuthorId from AUTHOR where AuthorName=@AuthorName),
	(select CategoryId from CATEGORY where CategoryName=@CategoryName)
	)


if not exists (select * from LIBRARY where LibraryName = 'Telemark University College')
	INSERT INTO LIBRARY (LibraryName) VALUES ('Telemark University College')

INSERT INTO BOOK_LIBRARY(BookId, LibraryId) 
VALUES 
(
(select BookId from BOOK where Title = @Title),
(select LibraryId from LIBRARY where LibraryName='Telemark University College')
)
GO


	
/**********************************************************************************************/
/*DeleteBook.sp.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'DeleteBook' 
	   AND 	  type = 'P')
	DROP PROCEDURE DeleteBook
GO

CREATE PROCEDURE DeleteBook
@BookId int
AS


delete from RATING where BookId=@BookId

delete from LOAN where BookId=@BookId

delete from BOOK_LIBRARY where BookId=@BookId

delete from BOOK where BookId=@BookId


GO


	
/**********************************************************************************************/
/*DeliverBook.sp.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'DeliverBook' 
	   AND 	  type = 'P')
	DROP PROCEDURE DeliverBook
GO

CREATE PROCEDURE DeliverBook
@BookId int,
@CustomerId int,
@LibraryId int
AS


update LOAN
set
Submitted=1,
SubmittedDate=getdate()
where BookId = @BookId
and CustomerId=@CustomerId
and LibraryId = @LibraryId 


update BOOK_LIBRARY
set OnLoan=0
where BookId = @BookId
and LibraryId = @LibraryId 

GO


	
/**********************************************************************************************/
/*RateBook.sp.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'RateBook' 
	   AND 	  type = 'P')
	DROP PROCEDURE RateBook
GO

CREATE PROCEDURE RateBook
@BookId int,
@CustomerId int,
@Score float,
@Comment varchar(1000)
AS


insert into RATING (BookId, CustomerId, RatingDate, Score, Comment)
values (@BookId, @CustomerId, GETDATE(), @Score, @Comment)



GO


	
/**********************************************************************************************/
/*RentBook.sp.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'RentBook' 
	   AND 	  type = 'P')
	DROP PROCEDURE RentBook
GO

CREATE PROCEDURE RentBook
@BookId int,
@CustomerId int,
@LibraryId int
AS


insert into LOAN (BookId, CustomerId, LibraryId, LoanDate, Deadline)
values (@BookId, @CustomerId, @LibraryId, getdate(), getdate()+30)


update BOOK_LIBRARY
set OnLoan=1
where BookId = @BookId
and LibraryId = @LibraryId 

GO


	
/**********************************************************************************************/
/*UpdateBook.sp.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'UpdateBook' 
	   AND 	  type = 'P')
	DROP PROCEDURE UpdateBook
GO

CREATE PROCEDURE UpdateBook
@BookId int,
@Title varchar(50),
@ISBN varchar(20),
@PublisherName varchar(50),
@AuthorName varchar(50),
@CategoryName varchar(50)
AS



--execute UpdateeBook BookId, 'Title', 'Author', 'Publisher', 'ISBN', 'Category'

if not exists (select * from CATEGORY where CategoryName = @CategoryName)
	INSERT INTO CATEGORY (CategoryName) VALUES (@CategoryName)

if not exists (select * from AUTHOR where AuthorName = @AuthorName)
	INSERT INTO AUTHOR (AuthorName) VALUES (@AuthorName)

if not exists (select * from PUBLISHER where PublisherName = @PublisherName)
	INSERT INTO PUBLISHER (PublisherName) VALUES (@PublisherName)



UPDATE BOOK SET
Title = @Title, 
ISBN = @ISBN, 
PublisherId = (select PublisherId from PUBLISHER where PublisherName=@PublisherName), 
AuthorId = (select AuthorId from AUTHOR where AuthorName=@AuthorName), 
CategoryId = (select CategoryId from CATEGORY where CategoryName=@CategoryName) 
	 
WHERE BookId = @BookId

GO


	
/**********************************************************************************************/
/*CalcAvgScore.trigger.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'CalcAvgScore' 
	   AND 	  type = 'TR')
	DROP TRIGGER CalcAvgScore
GO

CREATE TRIGGER CalcAvgScore ON RATING
FOR UPDATE, INSERT, DELETE

AS
DECLARE
@BookId int,
@AverageRating float


select @BookId = BookId from INSERTED 


select @AverageRating = AVG(Score) from RATING where BookId = @BookId


update BOOK set AverageRating = @AverageRating where BookId = @BookId

GO

	
/**********************************************************************************************/
/*Insert Data into Tables.sql*/
/**********************************************************************************************/
/*--------------------------------------------------------------------------------------------
Last Updated Date:	2012.03.22
Last Updated By:	hans.p.halvorsen@hit.no
Description:		Insert Data into Book Library Tables

(C) Hans-Petter Halvorsen
-------------------------------------------------------------------------------------------*/

/*
CUSTOMER
LIBRARY
CATEGORY
AUTHOR
PUBLISHER
BOOK

*/


--CUSTOMER ----------------------------------
INSERT INTO CUSTOMER (CustomerName) 
VALUES ('Bill Clinton')
GO
INSERT INTO CUSTOMER (CustomerName) 
VALUES ('Jens Stoltenberg')
GO
INSERT INTO CUSTOMER (CustomerName) 
VALUES ('Barak Obama') 
GO


--LIBRARY ----------------------------------
INSERT INTO LIBRARY (LibraryName) 
VALUES ('Telemark University College')
GO
INSERT INTO LIBRARY (LibraryName) 
VALUES ('University of Oslo')
GO
INSERT INTO LIBRARY (LibraryName) 
VALUES ('NTNU')
GO


--CATEGORY ----------------------------------
INSERT INTO CATEGORY (CategoryName) 
VALUES ('Science')
GO
INSERT INTO CATEGORY (CategoryName) 
VALUES ('Programming')
GO
INSERT INTO CATEGORY (CategoryName) 
VALUES ('Novel')
GO


--AUTHOR ----------------------------------
INSERT INTO AUTHOR (AuthorName) 
VALUES ('Knut Hamsun')
GO
INSERT INTO AUTHOR (AuthorName) 
VALUES ('Gilbert Strang')
GO
INSERT INTO AUTHOR (AuthorName) 
VALUES ('J.R.R Tolkien')
GO
INSERT INTO AUTHOR (AuthorName) 
VALUES ('Dorf Bishop')
GO


--PUBLISHER ----------------------------------
INSERT INTO PUBLISHER (PublisherName) 
VALUES ('Prentice Hall')
GO
INSERT INTO PUBLISHER (PublisherName) 
VALUES ('Wiley')
GO
INSERT INTO PUBLISHER (PublisherName) 
VALUES ('McGraw-Hill')
GO



--BOOK ----------------------------------
INSERT INTO BOOK (Title, ISBN, PublisherId, AuthorId, CategoryId) 
VALUES 
(
'Introduction to Linear Algebra',
'0-07-066781-0', 
(select PublisherId from PUBLISHER where PublisherName='Prentice Hall'),
(select AuthorId from AUTHOR where AuthorName='Gilbert Strang'),
(select CategoryId from CATEGORY where CategoryName='Science')
)
GO

INSERT INTO BOOK (Title, ISBN, PublisherId, AuthorId, CategoryId) 
VALUES 
(
'Modern Control System',
'1-08-890781-0',  
(select PublisherId from PUBLISHER where PublisherName='Wiley'),
(select AuthorId from AUTHOR where AuthorName='Dorf Bishop'),
(select CategoryId from CATEGORY where CategoryName='Programming')
)
GO

INSERT INTO BOOK (Title, ISBN, PublisherId, AuthorId, CategoryId) 
VALUES 
(
'The Lord of the Rings',
'2-09-066556-2',  
(select PublisherId from PUBLISHER where PublisherName='McGraw-Hill'),
(select AuthorId from AUTHOR where AuthorName='J.R.R Tolkien'),
(select CategoryId from CATEGORY where CategoryName='Novel')
)
GO


	

--BOOK_LIBRARY ----------------------------------
INSERT INTO BOOK_LIBRARY(BookId, LibraryId) 
VALUES 
(
(select BookId from BOOK where Title ='Introduction to Linear Algebra'),
(select LibraryId from LIBRARY where LibraryName='Telemark University College')
)
GO

INSERT INTO BOOK_LIBRARY(BookId, LibraryId) 
VALUES 
(
(select BookId from BOOK where Title ='Modern Control System'),
(select LibraryId from LIBRARY where LibraryName='Telemark University College')
)
GO

INSERT INTO BOOK_LIBRARY(BookId, LibraryId) 
VALUES 
(
(select BookId from BOOK where Title ='The Lord of the Rings'),
(select LibraryId from LIBRARY where LibraryName='Telemark University College')
)
GO


--CHAPTER ----------------------------------
INSERT INTO CHAPTER (ChapterNumber, ChapterName, BookId) 
VALUES 
(
1,
'Chapter 1', 
(select BookId from BOOK where Title ='Introduction to Linear Algebra')
)
GO

INSERT INTO CHAPTER (ChapterNumber, ChapterName, BookId) 
VALUES 
(
2,
'Chapter 2', 
(select BookId from BOOK where Title ='Introduction to Linear Algebra')
)
GO

INSERT INTO CHAPTER (ChapterNumber, ChapterName, BookId) 
VALUES 
(
3,
'Chapter 3', 
(select BookId from BOOK where Title ='Introduction to Linear Algebra')
)
GO


INSERT INTO CHAPTER (ChapterNumber, ChapterName, BookId) 
VALUES 
(
1,
'Chapter 1', 
(select BookId from BOOK where Title ='Modern Control System')
)
GO

INSERT INTO CHAPTER (ChapterNumber, ChapterName, BookId) 
VALUES 
(
2,
'Chapter 2', 
(select BookId from BOOK where Title ='Modern Control System')
)
GO

INSERT INTO CHAPTER (ChapterNumber, ChapterName, BookId) 
VALUES 
(
3,
'Chapter 3', 
(select BookId from BOOK where Title ='Modern Control System')
)
GO



INSERT INTO CHAPTER (ChapterNumber, ChapterName, BookId) 
VALUES 
(
1,
'Chapter 1', 
(select BookId from BOOK where Title ='The Lord of the Rings')
)
GO

INSERT INTO CHAPTER (ChapterNumber, ChapterName, BookId) 
VALUES 
(
2,
'Chapter 2', 
(select BookId from BOOK where Title ='The Lord of the Rings')
)
GO

INSERT INTO CHAPTER (ChapterNumber, ChapterName, BookId) 
VALUES 
(
3,
'Chapter 3', 
(select BookId from BOOK where Title ='The Lord of the Rings')
)
GO














