/*--------------------------------------------------------------------------------------------
Last Updated Date:	2012.03.23 
Last Updated By:	hans.p.halvorsen@hit.no
Description:		Library Management System

(C) Hans-Petter Halvorsen
-------------------------------------------------------------------------------------------*/

/*
LIBRARY
AUTHOR
PUBLISHER
CATEGORY
BOOK
BOOK_LIBRARY
CHAPTER
CUSTOMER
LOAN
RATING
*/

if not exists (select * from dbo.sysobjects where id = object_id(N'[LIBRARY]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [LIBRARY]
(
	[LibraryId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[LibraryName] [varchar](50) NOT NULL UNIQUE,
	[Description] [varchar](1000) NULL,
	[Address] [varchar](50) NULL,
	[Phone] [varchar](50) NULL,
	[PostCode] [varchar](50) NULL,
	[PostAddress] [varchar](50) NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[AUTHOR]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [AUTHOR]
(
	[AuthorId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[AuthorName] [varchar](50) NOT NULL UNIQUE,
	[Address] [varchar](50) NULL,
	[Phone] [varchar](50) NULL,
	[PostCode] [varchar](50) NULL,
	[PostAddress] [varchar](50) NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[PUBLISHER]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [PUBLISHER]
(
	[PublisherId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[PublisherName] [varchar](50) NOT NULL UNIQUE,
	[Description] [varchar](1000) NULL,
	[Address] [varchar](50) NULL,
	[Phone] [varchar](50) NULL,
	[PostCode] [varchar](50) NULL,
	[PostAddress] [varchar](50) NULL,
	[EMail] [varchar](50) NULL,	
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[CATEGORY]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [CATEGORY]
(
	[CategoryId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[CategoryName] [varchar](50) NOT NULL UNIQUE,
	[Description] [varchar](1000) NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[BOOK]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [BOOK]
(
	[BookId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[Title] [varchar](50) NOT NULL UNIQUE,
	[ISBN] [varchar](20) NOT NULL,
	[PublisherId] [int] NOT NULL FOREIGN KEY REFERENCES [PUBLISHER] ([PublisherId]),
	[AuthorId] [int] NOT NULL FOREIGN KEY REFERENCES [AUTHOR] ([AuthorId]),
	[CategoryId] [int] NOT NULL FOREIGN KEY REFERENCES [CATEGORY] ([CategoryId]),
	[Description] [varchar](1000) NULL,
	[Year] [date] NULL,
	[Edition] [int] NULL,
	[AverageRating] [float] NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[BOOK_LIBRARY]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [BOOK_LIBRARY]
(
	[BookLibraryId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[BookId] [int] NOT NULL FOREIGN KEY REFERENCES [BOOK] ([BookId]),
	[LibraryId] [int] NOT NULL FOREIGN KEY REFERENCES [LIBRARY] ([LibraryId]),
	[OnLoan] [bit] NOT NULL DEFAULT(0),
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[CHAPTER]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [CHAPTER]
(
	[ChapterId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[ChapterNumber] [int] NOT NULL,
	[ChapterName] [varchar](50) NOT NULL,
	[BookId] [int] NOT NULL FOREIGN KEY REFERENCES [BOOK] ([BookId]),
	[Description] [varchar](1000) NULL,
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[CUSTOMER]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [CUSTOMER]
(
	[CustomerId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[CustomerName] [varchar](50) NOT NULL,
	[Address] [varchar](50) NULL,
	[Phone] [varchar](50) NULL,
	[PostCode] [varchar](50) NULL,
	[PostAddress] [varchar](50) NULL,
	[EMail] [varchar](50) NULL,	
) 
GO


if not exists (select * from dbo.sysobjects where id = object_id(N'[LOAN]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [LOAN]
(
	[LoanId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[BookId] [int] NOT NULL FOREIGN KEY REFERENCES [BOOK] ([BookId]),
	[CustomerId] [int] NOT NULL FOREIGN KEY REFERENCES [CUSTOMER] ([CustomerId]),
	[LibraryId] [int] NOT NULL FOREIGN KEY REFERENCES [LIBRARY] ([LibraryId]),
	[LoanDate] [datetime] NOT NULL,
	[Deadline] [datetime] NULL,
	[Submitted] [bit] NULL,
	[SubmittedDate] [datetime] NULL,
) 
GO



if not exists (select * from dbo.sysobjects where id = object_id(N'[RATING]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
CREATE TABLE [RATING]
(
	[RatingId] [int] IDENTITY(1, 1) NOT NULL PRIMARY KEY,
	[BookId] [int] NOT NULL FOREIGN KEY REFERENCES [BOOK] ([BookId]),
	[CustomerId] [int] NOT NULL FOREIGN KEY REFERENCES [CUSTOMER] ([CustomerId]),
	[RatingDate] [datetime] NOT NULL,
	[Score] [float] NOT NULL,
	[Comment] [varchar](1000) NULL,
) 
GO



